from flask import Blueprint, render_template, flash
from flask import Response, send_from_directory 
import config, os, json
from collections import defaultdict


filemgr = Blueprint('fileshare', __name__, url_prefix='/')

# -----------------------------------------------------
# Function for returning File Tree for specific path
# -----------------------------------------------------
def directory_print(root, ignore_files=[], ignore_directories =[]):
    extr =  {}
    # Sets up ignored file names what will not be showed
    if len(ignore_files) == 0:
        ignore_files=['.paths.json']
    else:
        ignore_directories = ignore_files.append('.paths.json')
    
    # Sets up ignored Folders what will not be showed
    if len(ignore_directories) == 0:
        ignore_directories=['personal','secret','vault']

    try:
        items = json.load(open(os.path.join(config.share_folder, '.paths.json')))
    except Exception:
        items = ""

    for root, dirs, files in os.walk(root): 
        
        for f in files:
            if f not in ignore_files:
                full_path = os.path.join(root, f)
                path = full_path.split(config.share_folder,1)[1]
                path = path.replace("\\", "/")
                split_path = path.split('/')
                if len(list(filter(set(split_path).__contains__, ignore_directories))) < 1:
                    extr.update({path[1:]: 1})
                else:
                    extr.update({path[1:]: 0})
    
    if items != extr:
        with open(os.path.join(config.share_folder, '.paths.json'), 'w', encoding='utf-8') as f:
            json.dump(extr, f, ensure_ascii=False, indent=4)



@filemgr.route('/')
def fileshare():
    # Updates current file structure on page load
    directory_print(config.share_folder)
    sorted_files = defaultdict(list)
    path_list = []
    try:
        items = json.load(open(os.path.join(config.share_folder, '.paths.json')))
    except Exception as err:
        print(err)
        items={"":""}

    for key, arr in items.items():
        path_only = ('/').join(key.split('/')[:-1]) 
        path_list.append(path_only)
        file_name = key.split('/')[-1:]
        if arr == 1:
            sorted_files[path_only].append(file_name[0])  

    return render_template('index.html', filestructure=sorted_files,folders=set(path_list))

@filemgr.route('/down/<path:filename>')
def download(filename):
    # Downloads files
    try:
        items = json.load(open(os.path.join(config.share_folder, '.paths.json')))
    except Exception as err:
        print(err)

    existing = []    
    uploads = config.share_folder
    # Makes files available to be downloaded if they are in list
    # It makes even hidden files in list with arr==0 available
    # Path to hidden files must be known to be downloaded
    for key, arr in items.items():
        existing.append(key)

    if filename in existing:
        return send_from_directory(uploads, filename, as_attachment=True)
    else:
        flash("File not found!")
        return render_template('404.html', title = '404'), 404

@filemgr.route('/read/<path:filename>')
def readfile(filename):
    # Read files in fileshare folder
    extensions = ('.txt','.pdf','.jpg','.jpeg','.png','.doc','.docx','.ico','.html')
    try:
        items = json.load(open(os.path.join(config.share_folder, '.paths.json')))
    except Exception as err:
        print(err)

    existing = []    
    uploads = config.share_folder
    # Makes files available  to be read if they are in the list
    # It makes even hidden files in list with arr==0 available
    # Path to hidden files must be known to be read
    for key, arr in items.items():
        existing.append(key)

    if filename in existing: 
        if filename.endswith(extensions):
            with open(os.path.join(uploads, filename),"rb") as f:
                content = f.readlines()
            return Response(content, mimetype='text/plain')
        else:
            flash("File not readable!")
            return render_template('404.html', title = '404'), 404
    else:
        flash("File not found!")
        return render_template('404.html', title = '404'), 404